package com.muyu.data.source.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.muyu.data.source.domain.*;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.data.source.domain.req.DataSourceQueryReq;
import com.muyu.data.source.domain.req.DataSourceSaveReq;
import com.muyu.data.source.domain.req.DataSourceEditReq;
import com.muyu.data.source.service.DataSourceService;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 数据源Controller
 *
 * @author muyu
 * @date 2024-04-21
 */
@Api(tags = "数据源")
@RestController
@RequestMapping("/source")
public class DataSourceController extends BaseController {
    @Autowired
    private DataSourceService dataSourceService;

    /**
     * 查询数据源列表
     */
    @ApiOperation("获取数据源列表")
    @RequiresPermissions("dataSource:source:list")
    @GetMapping("/list")
    public Result<TableDataInfo<DataSource>> list(DataSourceQueryReq dataSourceQueryReq) {
        startPage();
        List<DataSource> list = dataSourceService.list(DataSource.queryBuild(dataSourceQueryReq));
        return getDataTable(list);
    }
    /**
     * post请求查询数据源列表
     */
    @PostMapping("/DataSourceList")
    public Result<List<DataSource>> postList(DataSourceQueryReq dataSourceQueryReq) {
        List<DataSource> list = dataSourceService.list(DataSource.queryBuild(dataSourceQueryReq));
        return Result.success(list);
    }

    /**
     * 导出数据源列表
     */
    @ApiOperation("导出数据源列表")
    @RequiresPermissions("dataSource:source:export")
    @Log(title = "数据源", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataSource dataSource) {
        List<DataSource> list = dataSourceService.list(dataSource);
        ExcelUtil<DataSource> util = new ExcelUtil<DataSource>(DataSource.class);
        util.exportExcel(response, list, "数据源数据");
    }

    /**
     * 获取数据源详细信息
     */
    @ApiOperation("获取数据源详细信息")
    @RequiresPermissions("dataSource:source:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<DataSource> getInfo(@PathVariable("id") Long id) {
        return Result.success(dataSourceService.getById(id));
    }

    /**
     * 新增数据源
     */
    @RequiresPermissions("dataSource:source:add")
    @Log(title = "数据源", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增数据源")
    public Result<String> add(@RequestBody DataSourceSaveReq dataSourceSaveReq) {
        return toAjax(dataSourceService.save(DataSource.saveBuild(dataSourceSaveReq)));
    }

    /**
     * 修改数据源
     */
    @RequiresPermissions("dataSource:source:edit")
    @Log(title = "数据源", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改数据源")
    public Result<String> edit(@PathVariable Long id, @RequestBody DataSourceEditReq dataSourceEditReq) {
        return toAjax(dataSourceService.updateById(DataSource.editBuild(id,dataSourceEditReq)));
    }

    /**
     * 删除数据源
     */
    @RequiresPermissions("dataSource:source:remove")
    @Log(title = "数据源", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除数据源")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(dataSourceService.removeBatchByIds(ids));
    }
    /**
     * 查询接入类型列表
     */
    @GetMapping("/findAccessType")
    public Result<List<AccessType>> findAccessType(){
        List<AccessType> accessTypeList=dataSourceService.findAccessType();
        return Result.success(accessTypeList);
    }

    /**
     * 测试连接
     * @param dataSourceSaveReq
     * @return
     */
    @PostMapping("/testConnection")
    public Result testConnection(@RequestBody DataSourceSaveReq dataSourceSaveReq){
        return dataSourceService.testConnection(dataSourceSaveReq);
    }

    /**
     * 根据数据源对象同步代码
     * @param dataSource
     * @return
     */
    @PostMapping("/synchronization")
    public Result synchronization(@RequestBody DataSource dataSource){
        return dataSourceService.synchronization(dataSource);
    }

    @GetMapping("/findAssetStructure")
    public Result<List<AssetStructure>> findAssetStructure(){
        return dataSourceService.findAssetStructure();
    }
    @GetMapping("/findInformationById")
    public Result<List<DatabaseTableInformation>> findInformationById(@RequestParam Integer id){
        return dataSourceService.findInformationById(id);
    }

    /**
     * 根据资产表id获取字段表信息
     * @param id
     * @return 字段列表
     */
    @GetMapping("/findDataBaseTableById")
    public Result<List<DataBaseTable>> findDataBaseTableById(@RequestParam Integer id){
        return dataSourceService.findDataBaseTableById(id);
    }

    /**
     * 根据名字查询数据库
     * @param name
     * @return
     */
    @GetMapping("/findDataBaseTableByName/{name}")
    public Result<List<DataBaseTable>> findDataBaseTableByName( @PathVariable String name){
        return dataSourceService.findDataBaseTableByName(name);
    }

    /**
     *  根据id查询数据库
     * @param id
     * @return
     */
    @GetMapping("/findDataBaseTable")
    public Result<DatabaseTableInformation> findDataBaseTable(@RequestParam Long id){
        return dataSourceService.findDataBaseTable(id);
    }

    /**
     * 根据资产id查询数据库信息表
     * @param id
     * @return 数据库信息列表
     */
    @GetMapping("/findDataBaseByAssetId")
    public Result<List<DatabaseTableInformation>> findDataBaseByAssetId(@RequestParam Integer id){
        return dataSourceService.findDataBaseByAssetId(id);
    }

    /**
     * 根据数据库信息表id查询字段表
     * @param id
     * @return 字段表列表
     */
    @GetMapping("/findDataBaseByInformationId")
    public Result<List<DataBaseTable>> findDataBaseByInformationId(@RequestParam Integer id){
        return dataSourceService.findDataBaseByInformationId(id);
    }

    @GetMapping("/dataBaseTableInformation")
    public Result<List<DatabaseTableInformation>> dataBaseTableInformation(){
        return dataSourceService.dataBaseTableInformation();
    }

    @PostMapping("/updateDatabaseTable")
    public Result updateDatabaseTable(@RequestBody DataBaseTable dataBaseTable){
        return dataSourceService.updateDatabaseTable(dataBaseTable);
    }

    @PostMapping("/findDataSourceByDataBaseId")
    public Result findDataSourceByDataBaseId(@RequestParam Integer id){
        return dataSourceService.findDataSourceByDataBaseId(id);
    }
}
